"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _logUpdate = _interopRequireDefault(require("log-update"));

var _lodash = _interopRequireDefault(require("lodash.throttle"));

var _createReconciler = _interopRequireDefault(require("./create-reconciler"));

var _createRenderer = _interopRequireDefault(require("./create-renderer"));

var _diffString = _interopRequireDefault(require("./diff-string"));

var _dom = require("./dom");

var _App = _interopRequireDefault(require("./components/App"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var _default = (node, options = {}) => {
  // Stream was passed instead of `options` object
  if (typeof options.write === 'function') {
    options = {
      stdout: options,
      stdin: process.stdin
    };
  }

  options = _objectSpread({
    stdout: process.stdout,
    stdin: process.stdin,
    debug: false
  }, options);
  const rootNode = (0, _dom.createNode)('root');
  const render = (0, _createRenderer.default)({
    terminalWidth: options.stdout.columns
  });

  const log = _logUpdate.default.create(options.stdout); // Ignore last render after unmounting a tree to prevent empty output before exit


  let ignoreRender = false; // Store last output to only rerender when needed

  let lastOutput = '';
  let lastStaticOutput = '';

  const onRender = () => {
    if (ignoreRender) {
      return;
    }

    const {
      output,
      staticOutput
    } = render(rootNode);

    if (options.debug) {
      options.stdout.write((staticOutput || '') + output);
      return;
    } // If <Static> part of the output has changed, calculate the difference
    // between the last <Static> output and log it to stdout.
    // To ensure static output is cleanly rendered before main output, clear main output first


    if (staticOutput && staticOutput !== lastStaticOutput) {
      log.clear();
      options.stdout.write((0, _diffString.default)(lastStaticOutput, staticOutput));
      log(output);
      lastStaticOutput = staticOutput;
    }

    if (output !== lastOutput) {
      log(output);
      lastOutput = output;
    }
  };

  const throttledRender = options.debug ? onRender : (0, _lodash.default)(onRender, 50, {
    leading: true,
    trailing: true
  });
  const reconciler = options.stdout._inkReconciler || (0, _createReconciler.default)(throttledRender);

  if (!options.stdout._ink) {
    options.stdout._ink = true;
    options.stdout._inkReconciler = reconciler;
    options.stdout._inkContainer = reconciler.createContainer(rootNode, false);
  }

  const tree = _react.default.createElement(_App.default, {
    stdin: options.stdin,
    stdout: options.stdout
  }, node);

  reconciler.updateContainer(tree, options.stdout._inkContainer);
  return () => {
    if (typeof throttledRender.cancel === 'function') {
      throttledRender.cancel();
      onRender();
      log.done();
    }

    ignoreRender = true;
    reconciler.updateContainer(null, options.stdout._inkContainer);
  };
};

exports.default = _default;